@@ -14,7 +14,7 @@ sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..',
 from optparse import make_option, OptionParser
 
 from xapiand import version
-from xapiand.server import server_run
+from xapiand.server import xapiand_run
 from xapiand.platforms import EX_FAILURE, detached
 
 help = "Starts a Xapiand server"
@@ -53,14 +53,14 @@ def detach(path, argv, logfile=None, pidfile=None,
            uid=None, gid=None, umask=0, working_directory=None, fake=False, **options):
     with detached(logfile, pidfile, uid, gid, umask, working_directory, fake):
         try:
-            args = [path] + list(argv)
+            args = list(argv)
             if logfile:
                 args.append('--logfile=%s' % logfile)
             if pidfile:
                 args.append('--pidfile=%s' % pidfile)
             os.execv(path, args)
         except Exception:
-            print >>sys.stderr, "Can't exec %r" % ' '.join(args)
+            print >>sys.stderr, "Can't exec %r" % ' '.join([path] + args)
         return EX_FAILURE
 
 
@@ -71,7 +71,7 @@ def run(logfile=None, pidfile=None, *argv, **options):
         pidfile = pidfile or 'xapiand.pid'
         sys.exit(detach(sys.argv[0], argv, logfile=logfile, pidfile=pidfile, **options))
     else:
-        sys.exit(server_run(logfile=logfile, pidfile=pidfile, **options))
+        sys.exit(xapiand_run(logfile=logfile, pidfile=pidfile, **options))
 
 
 def main():
