@@ -81,6 +81,7 @@ class Xapian(object):
         self._check_db()
 
         terms = kwargs.get('terms')
+        ranges = kwargs.get('ranges')
         partials = kwargs.get('partials')
         results_class = kwargs.get('results_class', XapianResults)
         query = search_parser(search if isinstance(search, dict) else 'FACETS ' + search)
@@ -89,6 +90,8 @@ class Xapian(object):
             query['facets'].extend(facets)
         if terms is not None:
             query['terms'] = terms
+        if ranges is not None:
+            query['ranges'] = ranges
         if partials is not None:
             query['partials'] = partials
         del query['first']
@@ -97,12 +100,14 @@ class Xapian(object):
         search = self._search(query, get_matches=False, get_data=False, get_terms=False, get_size=False)
         return results_class(search.results)
 
-    def terms(self, search=None, terms=None, partials=None, offset=None, limit=None, order_by=None, results_class=XapianResults):
+    def terms(self, search=None, terms=None, ranges=None, partials=None, offset=None, limit=None, order_by=None, results_class=XapianResults):
         self._check_db()
         query = search_parser(search if isinstance(search, dict) else 'TERMS ' + search)
         del query['facets']
         if terms is not None:
             query['terms'] = terms
+        if ranges is not None:
+            query['ranges'] = ranges
         if partials is not None:
             query['partials'] = partials
         if offset is not None:
@@ -114,13 +119,15 @@ class Xapian(object):
         search = self._search(query, get_matches=True, get_data=False, get_terms=True, get_size=True)
         return results_class(search.results)
 
-    def find(self, search=None, facets=None, terms=None, partials=None, offset=None, limit=None, order_by=None, results_class=XapianResults):
+    def find(self, search=None, facets=None, terms=None, ranges=None, partials=None, offset=None, limit=None, order_by=None, results_class=XapianResults):
         self._check_db()
         query = search_parser(search)
         if facets is not None:
             query['facets'] = facets
         if terms is not None:
             query['terms'] = terms
+        if ranges is not None:
+            query['ranges'] = ranges
         if partials is not None:
             query['partials'] = partials
         if offset is not None:
@@ -132,13 +139,15 @@ class Xapian(object):
         search = self._search(query, get_matches=True, get_data=False, get_terms=False, get_size=True)
         return results_class(search.results)
 
-    def search(self, search=None, facets=None, terms=None, partials=None, offset=None, limit=None, order_by=None, results_class=XapianResults):
+    def search(self, search=None, facets=None, terms=None, ranges=None, partials=None, offset=None, limit=None, order_by=None, results_class=XapianResults):
         self._check_db()
         query = search_parser(search)
         if facets is not None:
             query['facets'] = facets
         if terms is not None:
             query['terms'] = terms
+        if ranges is not None:
+            query['ranges'] = ranges
         if partials is not None:
             query['partials'] = partials
         if offset is not None:
@@ -150,12 +159,14 @@ class Xapian(object):
         search = self._search(query, get_matches=True, get_data=True, get_terms=False, get_size=True)
         return results_class(search.results)
 
-    def count(self, search=None, terms=None, partials=None):
+    def count(self, search=None, terms=None, ranges=None, partials=None):
         if search or terms or partials:
             query = search_parser(search)
             del query['facets']
             if terms is not None:
                 query['terms'] = terms
+            if ranges is not None:
+                query['ranges'] = ranges
             if partials is not None:
                 query['partials'] = partials
             del query['first']
