@@ -11,9 +11,9 @@ OFFSET_RE = re.compile(r'\bOFFSET\s+(\d+)\b', re.IGNORECASE)
 LIMIT_RE = re.compile(r'\bLIMIT\s+(\d+)\b', re.IGNORECASE)
 ORDER_BY_RE = re.compile(r'\bORDER\s+BY\s+([-+_a-zA-Z0-9, ]+?)(\s+ASC\b|\s+DESC\b|$)', re.IGNORECASE)
 FACETS_RE = re.compile(r'\bFACETS\s+(\d+)?([_a-zA-Z0-9, ]+)\b', re.IGNORECASE)
-PARTIAL_RE = re.compile(r'\bPARTIAL\s+([_a-zA-Z0-9, *]+)\b', re.IGNORECASE)
-SEARCH_RE = re.compile(r'\bSEARCH\s+(.+)', re.IGNORECASE)
-TERMS_RE = re.compile(r'\bTERMS\s+(.+)', re.IGNORECASE)
+PARTIAL_RE = re.compile(r'\bPARTIAL\s+(.*)', re.IGNORECASE)
+SEARCH_RE = re.compile(r'\bSEARCH\s+(.*)', re.IGNORECASE)
+TERMS_RE = re.compile(r'\bTERMS\s+(.*)', re.IGNORECASE)
 
 CMDS_RE = re.compile(r'\b(OFFSET|LIMIT|ORDER\s+BY|FACETS|PARTIAL|SEARCH|TERMS)\s', re.IGNORECASE)
 
@@ -75,7 +75,7 @@ def index_parser(document):
         return ">> ERR: [400] You must provide 'data' to index"
     try:
         document_data = json.dumps(document_data, ensure_ascii=False).encode('safe-utf-8')
-    except:
+    except Exception:
         return ">> ERR: [400] 'data' must be a valid json serializable object"
 
     default_language = document.pop('language', None)
@@ -174,14 +174,14 @@ def search_parser(query_string):
     first = 0
     partials = []
     maxitems = 10000
-    check_at_least = 10000
+    check_at_least = 0
     sort_by = None
     sort_by_reversed = None
-    search = None
+    search = []
     facets = None
-    terms = None
+    terms = []
 
-    query_string = 'SEARCH %s' % (query_string or '')
+    query_string = " SEARCH %s " % (query_string or '')
     query_re = r''.join('(%s.*)' % s for s in CMDS_RE.findall(query_string))
 
     for string in re.search(query_re, query_string).groups():
@@ -218,7 +218,7 @@ def search_parser(query_string):
             facets = SPLIT_RE.split(match.group(2).strip())
             if facets:
                 if match.group(1):
-                    check_at_least = max(min(int(match.group(1)), check_at_least), 0)
+                    check_at_least = max(min(int(match.group(1)), 10000), 0)
             else:
                 check_at_least = 0
             # print "facets:", facets
@@ -228,7 +228,9 @@ def search_parser(query_string):
         match = PARTIAL_RE.search(string)
         if match:
             string = PARTIAL_RE.sub('', string)
-            partials.append(match.group(1).strip())
+            v = match.group(1).strip()
+            if v:
+                partials.append(v)
             # print "partials:", partials
             continue
 
@@ -236,7 +238,9 @@ def search_parser(query_string):
         match = TERMS_RE.search(string)
         if match:
             string = TERMS_RE.sub('', string)
-            terms = match.group(1).strip()
+            v = match.group(1).strip()
+            if v:
+                terms.append(v)
             # print "terms:", terms
             continue
 
@@ -244,7 +248,9 @@ def search_parser(query_string):
         match = SEARCH_RE.search(string)
         if match:
             string = SEARCH_RE.sub('', string)
-            search = match.group(1).strip()
+            v = match.group(1).strip()
+            if v:
+                search.append(v)
             # print "search:", search
             continue
 
