@@ -6,8 +6,8 @@ import logging
 import xapian
 
 from . import json
-from .core import xapian_reopen, get_slot
-from .serialise import normalize
+from .core import xapian_reopen, get_slot, expand_terms, find_terms, DOCUMENT_CUSTOM_TERM_PREFIX
+from .serialise import normalize, serialise_value
 from .exceptions import XapianError
 
 MAX_DOCS = 10000
@@ -27,41 +27,92 @@ class Search(object):
         self.dead = dead
 
         # SEARCH
-        qp = xapian.QueryParser()
-        qp.set_database(database)
+        queryparser = xapian.QueryParser()
+        queryparser.set_database(database)
 
         query = None
 
+        prefixes = set()
+
+        def add_prefixes(string):
+            for term, term_field, terms in find_terms(string):
+                if term_field and term_field not in prefixes:
+                    prefix = '%s%s' % (DOCUMENT_CUSTOM_TERM_PREFIX, get_slot(term_field))
+                    if term_field.lower() == term_field:
+                        queryparser.add_prefix(term_field, prefix)
+                    else:
+                        queryparser.add_boolean_prefix(term_field, prefix)
+                    prefixes.add(term_field)
+
         # Build final query:
         search = obj.get('search')
-        if search and search != '*':
+        if search:
+            if not isinstance(search, (tuple, list)):
+                search = [search]
+            search = " AND ".join("(%s)" % s for s in search if s)
+        if search and search != '(*)':
             try:
                 query = xapian.Query.unserialise(search)
             except xapian.InvalidArgumentError:
                 pass
             if not query:
-                search = normalize(search)
+                search = normalize(search).encode('utf-8')
+
+                ranges = obj.get('ranges')
+                if ranges:
+                    _ranges = set()
+                    for field, begin, end in ranges:
+                        field = field.encode('utf-8')
+                        if field not in _ranges:
+                            slot = get_slot(field)
+                            vrp = xapian.StringValueRangeProcessor(slot, field)
+                            queryparser.add_valuerangeprocessor(vrp)
+                            _ranges.add(field)
+                        if begin is None:
+                            begin = b''
+                        if end is None:
+                            end = b''
+                        rng1 = b'(%s:%s..%s)' % (field, begin, end)
+                        rng2 = b'(%s:%s..%s)' % (field, serialise_value(begin)[0], serialise_value(end)[0])
+                        if rng1 == rng2:
+                            _search = search
+                            if rng1 in search:
+                                search = None
+                        else:
+                            _search = search.replace(rng1, rng2)
+                        if search != _search:
+                            search = _search
+                        else:
+                            search += b' AND %s' % rng2
+
+                search = expand_terms(search)
+                add_prefixes(search)
+                flags = xapian.QueryParser.FLAG_DEFAULT | xapian.QueryParser.FLAG_WILDCARD | xapian.QueryParser.FLAG_PURE_NOT
                 try:
-                    query = qp.parse_query(search)
+                    query = queryparser.parse_query(search, flags)
                 except (xapian.NetworkError, xapian.DatabaseModifiedError):
                     database = xapian_reopen(database, data=self.data, log=self.log)
-                    qp.set_database(database)
-                    query = qp.parse_query(search)
+                    queryparser.set_database(database)
+                    query = queryparser.parse_query(search, flags)
 
         partials = obj.get('partials')
         if partials:
+            if not isinstance(partials, (tuple, list)):
+                partials = [partials]
             # Partials (for autocomplete) using FLAG_PARTIAL and OP_AND_MAYBE
             partials_query = None
             for partial in partials:
                 self.dead or 'alive'  # Raises DeadException when needed
                 partial = normalize(partial)
-                flags = xapian.QueryParser.FLAG_WILDCARD | xapian.QueryParser.FLAG_PARTIAL
+                partial = expand_terms(partial)
+                add_prefixes(partial)
+                flags = xapian.QueryParser.FLAG_PARTIAL
                 try:
-                    _partials_query = qp.parse_query(partial, flags)
+                    _partials_query = queryparser.parse_query(partial, flags)
                 except (xapian.NetworkError, xapian.DatabaseModifiedError):
                     database = xapian_reopen(database, data=self.data, log=self.log)
-                    qp.set_database(database)
-                    _partials_query = qp.parse_query(partial, flags)
+                    queryparser.set_database(database)
+                    _partials_query = queryparser.parse_query(partial, flags)
                 if partials_query:
                     partials_query = xapian.Query(
                         xapian.Query.OP_AND_MAYBE,
@@ -81,25 +132,30 @@ class Search(object):
 
         terms = obj.get('terms')
         if terms:
-            # Partials (for autocomplete) using FLAG_BOOLEAN and OP_AND
-            terms = normalize(terms)
-            try:
-                terms_query = qp.parse_query(terms, xapian.QueryParser.FLAG_BOOLEAN)
-            except (xapian.NetworkError, xapian.DatabaseModifiedError):
-                database = xapian_reopen(database, data=self.data, log=self.log)
-                qp.set_database(database)
-                terms_query = qp.parse_query(terms, xapian.QueryParser.FLAG_BOOLEAN)
-            if query:
-                query = xapian.Query(
-                    xapian.Query.OP_AND,
-                    query,
-                    terms_query,
-                )
-            else:
-                query = terms_query
+            if not isinstance(terms, (tuple, list)):
+                terms = [terms]
+            for term in terms:
+                term = normalize(term)
+                term = expand_terms(term)
+                add_prefixes(term)
+                flags = xapian.QueryParser.FLAG_BOOLEAN | xapian.QueryParser.FLAG_PURE_NOT
+                try:
+                    terms_query = queryparser.parse_query(term, flags)
+                except (xapian.NetworkError, xapian.DatabaseModifiedError):
+                    database = xapian_reopen(database, data=self.data, log=self.log)
+                    queryparser.set_database(database)
+                    terms_query = queryparser.parse_query(term, flags)
+                if query:
+                    query = xapian.Query(
+                        xapian.Query.OP_AND,
+                        query,
+                        terms_query,
+                    )
+                else:
+                    query = terms_query
 
         if not query:
-            if search == '*':
+            if search == '(*)':
                 query = xapian.Query('')
             else:
                 query = xapian.Query()
@@ -107,7 +163,7 @@ class Search(object):
         self.database = database
         self.query = query
         self.facets = obj.get('facets')
-        self.check_at_least = obj.get('check_at_least', MAX_DOCS)
+        self.check_at_least = obj.get('check_at_least', MAX_DOCS if self.facets else 0)
         self.maxitems = obj.get('maxitems', MAX_DOCS)
         self.first = obj.get('first', 0)
         self.sort_by = obj.get('sort_by')
@@ -183,7 +239,7 @@ class Search(object):
             self.database = xapian_reopen(self.database, data=self.data, log=self.log)
             try:
                 enquire = self.get_enquire(self.database)
-                matches = self.enquire.get_mset(self.first, maxitems, check_at_least)
+                matches = enquire.get_mset(self.first, maxitems, check_at_least)
             except (xapian.NetworkError, xapian.DatabaseError) as e:
                 raise XapianError(e)
 
@@ -231,7 +287,7 @@ class Search(object):
                     continue
                 try:
                     data = json.loads(data)
-                except:
+                except Exception:
                     data = base64.b64encode(data)
                 result.update({
                     'data': data,
