@@ -1,3 +1,3 @@
 from __future__ import absolute_import
 
-from .server import server_run  # NOQA
+from .server import xapiand_run  # NOQA
