@@ -116,6 +116,7 @@ class ClientReceiver(object):
         self.encoding = encoding
         self.encoding_errors = encoding_errors
         self.cmd_id = 0
+        self.activity = time.time()
 
     def close(self):
         self.closed = True
@@ -176,7 +177,7 @@ class ClientReceiver(object):
         self.socket_file.flush()
 
     def lineReceived(self, line):
-        pass
+        self.activity = time.time()
 
 
 class CommandServer(StreamServer):
@@ -201,24 +202,39 @@ class CommandServer(StreamServer):
         try:
             client.handle()
         finally:
-            self.clients.remove(client)
+            self.clients.discard(client)
             client.connectionLost()
 
-    def close(self):
+    def close(self, timeout=None):
         if self.closed:
-            self.log.debug("Closing clients...")
-            for client in self.clients:
-                client.client_socket._sock.close()
+            if timeout is None:
+                self.log.error("Forcing server shutdown (%s clients)...", len(self.clients))
         else:
-            self.log.debug("Closing listener socket. Waiting for clients...")
+            if timeout is None:
+                timeout = 10
+            self.log.warning("Hitting Ctrl+C again will terminate all running tasks!")
             super(CommandServer, self).close()
-            for client in self.clients:
-                client.close()
+
+        now = time.time()
+        clean = []
+        for client in self.clients:
+            if timeout is None or client._weak or now - client.activity > timeout:
+                try:
+                    client.client_socket._sock.close()
+                except AttributeError:
+                    pass
+                clean.append(client)
+        for client in clean:
+            self.clients.discard(client)
 
 
 class CommandReceiver(ClientReceiver):
     welcome = "# Welcome to the server! Type quit to exit."
 
+    def __init__(self, *args, **kwargs):
+        self._weak = False
+        super(CommandReceiver, self).__init__(*args, **kwargs)
+
     def connectionMade(self):
         self.log.info("New connection from %s:%d (%d open connections)" % (self.address[0], self.address[1], len(self.server.clients)))
         if self.welcome:
@@ -228,6 +244,7 @@ class CommandReceiver(ClientReceiver):
         self.log.info("Lost connection from %s:%d (%d open connections)" % (self.address[0], self.address[1], len(self.server.clients)))
 
     def lineReceived(self, line):
+        super(CommandReceiver, self).lineReceived(line)
         line = line.decode(self.encoding, self.encoding_errors)
         cmd, _, line = line.partition(' ')
         cmd = cmd.strip().lower()
@@ -256,6 +273,15 @@ class CommandReceiver(ClientReceiver):
         raise QuitCommand
     exit = quit
 
+    @command(internal=True)
+    def weak(self, line=''):
+        """
+        Makes the connection weak (closes when the server needs to)
+
+        """
+        self._weak = True
+        self.sendLine(">> OK")
+
     def _help(self, func, cmd):
         # Figure out indentation for docstring:
         doc = func.__doc__ or "No docs for %s." % cmd
