@@ -84,7 +84,7 @@ class FileQueue(object):
                 fname = '%s.%s' % (self.name, fnum)
                 os.unlink(fname)
                 # self.log.debug("Cleaned up file: %s", fname)
-            except:
+            except Exception:
                 pass
             fnum -= 1
 
