@@ -3,11 +3,12 @@ from __future__ import absolute_import, unicode_literals
 from multiprocessing.queues import Queue
 
 import logging
-logger = logging.getLogger(__name__)
 
 
 class MemoryQueue(Queue):
-    def __init__(self, name=None, log=None):
+    persistent = False
+
+    def __init__(self, name=None, log=logging):
         self.name = name
-        self.log = log or logger
+        self.log = log
         super(MemoryQueue, self).__init__()
