@@ -11,11 +11,12 @@ import threading
 import multiprocessing
 
 import gevent
-from gevent import queue
+from gevent import queue, monkey
+monkey.patch_all(thread=False)
 
 from .. import version, json
 from ..exceptions import InvalidIndexError, XapianError
-from ..core import DatabasesPool, xapian_database, xapian_close, xapian_commit, xapian_index, xapian_delete
+from ..core import DatabasesPool, xapian_database, xapian_cleanup, xapian_close, xapian_commit, xapian_index, xapian_delete, xapian_spawn, DATABASE_MAX_LIFE
 from ..platforms import create_pidlock
 from ..utils import parse_url, build_url, format_time
 from ..parser import index_parser, search_parser
@@ -43,7 +44,6 @@ LOG_FORMAT = "[%(asctime)s: %(levelname)s/%(processName)s:%(threadName)s] %(mess
 
 STOPPED = 0
 COMMIT_TIMEOUT = 1
-DATABASE_MAX_LIFE = 900  # stop writer adter 15 minutes of inactivity
 
 WRITERS_FILE = 'Xapian-Writers.db'
 QUEUE_WORKER_MAIN = 'Xapian-Worker'
@@ -59,12 +59,12 @@ class Obj(object):
         self.__dict__ = kwargs
 
 
-class XapianReceiver(CommandReceiver):
+class XapiandReceiver(CommandReceiver):
     welcome = "# Welcome to Xapiand! Type QUIT to exit, HELP for help."
 
     def __init__(self, *args, **kwargs):
         data = kwargs.pop('data', '.')
-        super(XapianReceiver, self).__init__(*args, **kwargs)
+        super(XapiandReceiver, self).__init__(*args, **kwargs)
         self._do_reopen = False
         self._do_init = set()
         self._inited = set()
@@ -78,10 +78,11 @@ class XapianReceiver(CommandReceiver):
             return
         if getattr(func, 'reopen', False) and self._do_reopen:
             self._reopen()
-        super(XapianReceiver, self).dispatch(func, line, command)
+        super(XapiandReceiver, self).dispatch(func, line, command)
 
     def _reopen(self, create=False, endpoints=None):
         endpoints = endpoints or self.active_endpoints
+
         with xapian_database(self.databases_pool, endpoints, writable=False, create=create, reopen=True, data=self.data, log=self.log):
             self._do_reopen = False
             self._do_init.add(endpoints)
@@ -215,7 +216,8 @@ class XapianReceiver(CommandReceiver):
                         self.sendLine(">> ERR: Unable to get results: %s" % e)
                         return
 
-                    self.sendLine("# DEBUG: Parsed query was: %s" % search.query)
+                    query_string = str(search.query)
+                    self.sendLine("# DEBUG: Parsed query was: %r" % query_string)
                     for warning in search.warnings:
                         self.sendLine("# WARNING: %s" % warning)
                     size = search.size
@@ -385,6 +387,13 @@ class XapianReceiver(CommandReceiver):
         self.sendLine(">> OK")
         self._init()
 
+    @command(internal=True)
+    def spawn(self, line=''):
+        time_, address = xapian_spawn(line, data=self.data, log=self.log)
+        server = "%s %s:%s" % (time_, address[0], address[1])
+        self.sendLine(">> OK: %s" % server)
+        return server
+
     @command(db=True)
     def endpoints(self, line=''):
         endpoints = self.active_endpoints or []
@@ -417,16 +426,16 @@ class XapianReceiver(CommandReceiver):
         self.sendLine(">> OK: %d active databases::\n%s" % (size, "\n".join(lines)))
 
 
-class XapianServer(CommandServer):
-    receiver_class = XapianReceiver
+class XapiandServer(CommandServer):
+    receiver_class = XapiandReceiver
 
     def __init__(self, *args, **kwargs):
         self.data = kwargs.pop('data', '.')
         self.databases_pool = kwargs.pop('databases_pool')
-        super(XapianServer, self).__init__(*args, **kwargs)
-        address = self.address[0] or '0.0.0.0'
+        super(XapiandServer, self).__init__(*args, **kwargs)
+        address = self.address[0] or '127.0.0.1'
         port = self.address[1]
-        self.log.info("Listening to %s:%s", address, port)
+        self.log.info("Xapiand Server Listening to %s:%s", address, port)
 
     def buildClient(self, client_socket, address):
         return self.receiver_class(self, client_socket, address, data=self.data, log=self.log)
@@ -597,65 +606,57 @@ def _writer_loop(databases, databases_pool, db, tq, commit_lock, timeouts, data,
     log.debug("Writer %s ended! ~ lived for %s", name, format_time(time.time() - start))
 
 
-def logging_run(loglevel, logfile, pidfile, log_queue):
-    logger = logging.getLogger()
-    logger.setLevel(loglevel)
-    if len(logger.handlers) < 1:
+def logger_run(loglevel, log_queue, logfile, pidfile):
+    log = logging.getLogger()
+    log.setLevel(loglevel)
+    if len(log.handlers) < 1:
         formatter = logging.Formatter(LOG_FORMAT)
         if logfile:
             outfile = logging.FileHandler(logfile)
             outfile.setFormatter(formatter)
-            logger.addHandler(outfile)
+            log.addHandler(outfile)
         if not pidfile:
             console = ColoredStreamHandler(sys.stderr)
             console.setFormatter(formatter)
-            logger.addHandler(console)
+            log.addHandler(console)
 
+    log.warning("Starting Xapiand Logger (pid:%s)", os.getpid())
+
+    quit = 0
     while True:
         try:
             record = log_queue.get()
             if record is None:  # We send this as a sentinel to tell the listener to quit.
                 break
-            logger.handle(record)  # No level or filter logic applied - just do it!
+            log.handle(record)  # No level or filter logic applied - just do it!
         except (KeyboardInterrupt, SystemExit):
+            if quit >= 3:
+                raise
+            quit += 1
             continue
         except:
             import traceback
             traceback.print_exc(file=sys.stderr)
 
+    log.warning("Xapiand Logger ended! (pid:%s)", os.getpid())
 
-def server_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask=0,
-               working_directory=None, verbosity=2, commit_slots=None, commit_timeout=None,
-               port=None, queue=None, **options):
+
+def server_run(loglevel, log_queue, address, port, commit_slots, commit_timeout, data):
     global PQueue, STOPPED
 
-    loglevel = ['ERROR', 'WARNING', 'INFO', 'DEBUG'][3 if verbosity == 'v' else int(verbosity)]
-
-    log_queue = multiprocessing.Queue()
-    logging_job = multiprocessing.Process(target=logging_run, args=(loglevel, logfile, pidfile, log_queue))
-    logging_job.start()
-
     log = logging.getLogger()
     log.addHandler(QueueHandler(log_queue))
     log.setLevel(loglevel)
 
-    if pidfile:
-        create_pidlock(pidfile)
+    commit_slots = commit_slots or multiprocessing.cpu_count()
 
     if commit_timeout is None:
         commit_timeout = COMMIT_TIMEOUT
     timeout = min(max(int(round(commit_timeout * 0.3)), 1), 3)
 
-    commit_slots = commit_slots or multiprocessing.cpu_count()
-
     PQueue = AVAILABLE_QUEUES.get(queue) or AVAILABLE_QUEUES[None]
     mode = "with multiple threads and %s commit slots using %s" % (commit_slots, PQueue.__name__)
-    log.warning("Starting Xapiand %s %s [%s] (pid:%s)", version, mode, loglevel, os.getpid())
-
-    if ':' in port:
-        listen = port
-    else:
-        listen = ':%s' % port
+    log.warning("Starting Xapiand Server %s %s [%s] (pid:%s)", version, mode, loglevel, os.getpid())
 
     commit_lock = threading.Semaphore(commit_slots)
     timeouts = Obj(
@@ -668,40 +669,13 @@ def server_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask=
     databases_pool = DatabasesPool()
     databases = {}
 
-    server = XapianServer(listen, databases_pool=databases_pool, data=data, log=log)
+    xapian_server = XapiandServer((address, port), databases_pool=databases_pool, data=data, log=log)
 
-    def _server_stop(sig=None):
-        global STOPPED
-
-        now = time.time()
-
-        if sig == signal.SIGINT:
-            log.warning("Hitting Ctrl+C again will terminate all running tasks!")
-        elif sig:
-            log.warning("Sending the signal again will terminate all running tasks! (%s)", sig)
-
-        stopped = STOPPED
-        PQueue.STOPPED = STOPPED = now
-
-        if sig:
-            if stopped:
-                if now - stopped < 0.5:
-                    log.error("Killing process!...")
-                    sys.exit(-1)
-                    return
-                if now - stopped > 2:
-                    log.error("Forcing shutdown...")
-                    server.close()
-            else:
-                log.info("Warm shutdown... (%d open connections)", len(server.clients))
-                server.close()
-
-    gevent.signal(signal.SIGQUIT, _server_stop, signal.SIGQUIT)
-    gevent.signal(signal.SIGTERM, _server_stop, signal.SIGTERM)
-    gevent.signal(signal.SIGINT, _server_stop, signal.SIGINT)
+    gevent.signal(signal.SIGTERM, xapian_server.close)
+    gevent.signal(signal.SIGINT, xapian_server.close)
 
     log.debug("Starting server...")
-    server.start()
+    xapian_server.start()
 
     pq = get_queue(name=QUEUE_WORKER_MAIN, log=log)
 
@@ -735,8 +709,8 @@ def server_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask=
     log.info("Waiting for commands...")
     msg = None
     timeout = timeouts.timeout
-    while not STOPPED:
-        databases_pool.cleanup(DATABASE_MAX_LIFE, data=data, log=log)
+    while not xapian_server.closed:
+        xapian_cleanup(databases_pool, DATABASE_MAX_LIFE, data=data, log=log)
         try:
             msg = MAIN_QUEUE.get(True, timeout)
         except Queue.Empty:
@@ -761,10 +735,13 @@ def server_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask=
                 except Queue.Full:
                     log.error("Cannot send command to queue! (2)")
 
-    _server_stop()
+    log.debug("Waiting for connected clients to disconnect...")
+    while True:
+        if gevent.wait(timeout=5):
+            break
+        xapian_server.close(10)
 
-    log.debug("Waiting for server to stop...")
-    gevent.wait()  # Wait for worker
+    PQueue.STOPPED = STOPPED = time.time()
 
     if PQueue.persistent:
         with open(writers_file, 'wt') as epfile:
@@ -783,7 +760,60 @@ def server_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask=
     for t, tq in databases.values():
         t.join()
 
-    log.warning("Xapiand ended! (pid:%s)", os.getpid())
+    xapian_cleanup(databases_pool, 0, data=data, log=log)
 
-    log_queue.put_nowait(None)
-    logging_job.join()
+    log.warning("Xapiand Server ended! (pid:%s)", os.getpid())
+
+
+def xapiand_run(data=None, logfile=None, pidfile=None, uid=None, gid=None, umask=0,
+        working_directory=None, verbosity=2, commit_slots=None, commit_timeout=None,
+        port=None, queue=None, **options):
+    logger_job = server_job = None
+
+    if pidfile:
+        create_pidlock(pidfile)
+
+    address, _, port = port.partition(':')
+    if not port:
+        port, address = address, ''
+    port = int(port)
+
+    loglevel = ['ERROR', 'WARNING', 'INFO', 'DEBUG'][3 if verbosity == 'v' else int(verbosity)]
+    log_queue = multiprocessing.Queue()
+
+    logger_job = multiprocessing.Process(
+        name="LoggerProcess",
+        target=logger_run,
+        args=(loglevel, log_queue, logfile, pidfile),
+    )
+    logger_job.start()
+
+    server_job = multiprocessing.Process(
+        name="ServerProcess",
+        target=server_run,
+        args=(loglevel, log_queue, address, port, commit_slots, commit_timeout, data),
+    )
+    server_job.start()
+
+    quit = 0
+    while True:
+        try:
+            if server_job:
+                server_job.join()
+
+            log_queue.put_nowait(None)
+            logger_job.join()
+
+            break
+        except (KeyboardInterrupt, SystemExit):
+            if quit >= 3:
+                if server_job:
+                    server_job.terminate()
+
+                logger_job.terminate()
+                raise
+            quit += 1
+            continue
+        except:
+            import traceback
+            traceback.print_exc(file=sys.stderr)
