@@ -34,7 +34,8 @@ def parse_url(url):
     hostname = unquote(parts.hostname or '')
     path = unquote(parts.path or '')
     if parts.scheme:
-        path = path.lstrip('/')
+        if path[0] == '/':
+            path = path[1:]
         scheme = parts.scheme
         if scheme == 'file':
             path, hostname = hostname + '/' + path, ''
@@ -99,27 +100,27 @@ def isoparse(ds):
             offset = ds[26:].replace(':', '')
             delta = datetime.timedelta(hours=int(offset[:-2]), minutes=int(offset[-2:]))
             dt = dt.replace(tzinfo=tzoffset(None, int(delta.total_seconds())))
-        except:
+        except Exception:
             pass
-    except:
+    except Exception:
         try:
             dt = datetime.datetime.strptime(ds[:19].replace(' ', 'T'), '%Y-%m-%dT%H:%M:%S')
             try:
                 offset = ds[19:].replace(':', '')
                 delta = datetime.timedelta(hours=int(offset[:-2]), minutes=int(offset[-2:]))
                 dt = dt.replace(tzinfo=tzoffset(None, int(delta.total_seconds())))
-            except:
+            except Exception:
                 pass
-        except:
+        except Exception:
             try:
                 dt = datetime.datetime.strptime(ds[:15], '%H:%M:%S.%f')
-            except:
+            except Exception:
                 try:
                     dt = datetime.datetime.strptime(ds[:8], '%H:%M:%S')
-                except:
+                except Exception:
                     try:
                         dt = datetime.datetime.strptime(ds[:8], '%H:%M')
-                    except:
+                    except Exception:
                         dt = datetime.datetime.strptime(ds[:10], '%Y-%m-%d')
     return dt
 
